@@ -3,6 +3,7 @@ import json
 import logging
 import urlparse
 
+from django.contrib.auth.models import User
 from django.core.files.base import ContentFile
 from django.db import connection
 from django.db import IntegrityError, transaction
@@ -113,33 +114,66 @@ class DatabaseJobSource(object):
             params['server'] = urlparse.urlunsplit(parsed)
         return json_data
 
+    def _getHealthCheckJobForBoard(self, device):
+        job_json = device.device_type.health_check_job
+        if not job_json:
+            self.logger.error(
+                "no job_json in getHealthCheckJobForBoard for %r", device)
+            return None
+        else:
+            user = User.objects.get(username='lava-health')
+            job_data = json.loads(job_json)
+            job_name = job_data.get('description')
+            job = TestJob(
+                definition=job_json, submitter=user, description=job_name,
+                health_check=True)
+            job.save()
+            return job
+
+    def _getJobFromQueue(self, device):
+        jobs_for_device = TestJob.objects.all().filter(
+            Q(requested_device=device)
+            | Q(requested_device_type=device.device_type),
+            status=TestJob.SUBMITTED)
+        jobs_for_device = jobs_for_device.extra(
+            select={
+                'is_targeted': 'requested_device_id is not NULL',
+                },
+            where=[
+                # In human language, this is saying "where the number of
+                # tags that are on the job but not on the device is 0"
+                '''(select count(*) from lava_scheduler_app_testjob_tags
+                     where testjob_id = lava_scheduler_app_testjob.id
+                       and tag_id not in (select tag_id
+                                            from lava_scheduler_app_device_tags
+                                           where device_id = '%s')) = 0'''
+                % device.hostname,
+                ],
+            order_by=['-is_targeted', 'submit_time'])
+        jobs = jobs_for_device[:1]
+        if jobs:
+            return jobs[0]
+        else:
+            return None
+
     def getJobForBoard_impl(self, board_name):
         while True:
             device = Device.objects.get(hostname=board_name)
             if device.status != Device.IDLE:
                 return None
-            jobs_for_device = TestJob.objects.all().filter(
-                Q(requested_device=device)
-                | Q(requested_device_type=device.device_type),
-                status=TestJob.SUBMITTED)
-            jobs_for_device = jobs_for_device.extra(
-                select={
-                    'is_targeted': 'requested_device_id is not NULL',
-                    },
-                where=[
-                    # In human language, this is saying "where the number of
-                    # tags that are on the job but not on the device is 0"
-                    '''(select count(*) from lava_scheduler_app_testjob_tags
-                         where testjob_id = lava_scheduler_app_testjob.id
-                           and tag_id not in (select tag_id
-                                                from lava_scheduler_app_device_tags
-                                               where device_id = '%s')) = 0'''
-                    % device.hostname,
-                    ],
-                order_by=['-is_targeted', 'submit_time'])
-            jobs = jobs_for_device[:1]
-            if jobs:
-                job = jobs[0]
+            if not device.device_type.health_check_job:
+                run_health_check = False
+            elif device.health_status == Device.HEALTH_UNKNOWN:
+                run_health_check = True
+            elif not device.last_health_report_job:
+                run_health_check = True
+            else:
+                run_health_check = device.last_health_report_job.end_time < datetime.datetime.now() - datetime.timedelta(days=1)
+            if run_health_check:
+                job = self._getHealthCheckJobForBoard(device)
+            else:
+                job = self._getJobFromQueue(device)
+            if job:
                 DeviceStateTransition.objects.create(
                     created_by=None, device=device, old_state=device.status,
                     new_state=Device.RUNNING, message=None, job=job).save()
@@ -214,6 +248,15 @@ class DatabaseJobSource(object):
         DeviceStateTransition.objects.create(
             created_by=None, device=device, old_state=old_device_status,
             new_state=device.status, message=None, job=job).save()
+
+        if job.health_check is True:
+            device.last_health_report_job = job
+            if job.status == TestJob.INCOMPLETE:
+                device.health_status = Device.HEALTH_FAIL
+                device.put_into_maintenance_mode(None, "Health Check Job Failed")
+            elif job.status == TestJob.COMPLETE:
+                device.health_status = Device.HEALTH_PASS
+
         job.end_time = datetime.datetime.utcnow()
         token = job.submit_token
         job.submit_token = None
